//
//  DSMailDelivery.h
//  MailDelivery
//
//  Created by darkshadow on 8/12/07.
//  Copyright 2007 Night Productions. All rights reserved.
//

#import <Cocoa/Cocoa.h>

typedef enum _DSDeliveryStatus {
	DSDeliveryOpeningConnection = 0,
	DSDeliveryCommunicatingWithServer,
	DSDeliveryServerError,
	DSDeliverySendingData,
	DSDeliveryClosingConnection,
	DSDeliveryIdle
} DSDeliveryStatus;

extern NSString *DSMailDeliveryInfoSent;
extern NSString *DSMailDeliveryInfoTotal;
extern NSString *DSMailDeliveryInfoServerCode;
extern NSString *DSMailDeliveryInfoServerError;
extern NSString *DSMailDeliveryServerURLKey;
extern NSString *DSMailDeliveryDisplayKey;
extern NSString *DSMailDeliveryUserKey;

@interface DSMailDelivery : NSObject {
	@private
	NSString *toAddress;
	NSString *toName;
	NSString *fromAddress;
	NSString *fromName;
	NSString *subject;
	NSURL *serverURL;
	NSDictionary *smtpInfo;
	NSMutableArray *attachments;
	NSMutableArray *headers;
	NSData *dataWaitingToSend;
	NSTextView *consoleTV;
	id body;
	IBOutlet id delegate;
	int server;
	int numberOfBoundaries;
	CFSocketRef serverSocket;
	DSDeliveryStatus deliveryStatus;
	unsigned int flags;
	unsigned long long totalDeliveryBytes;
	unsigned long long writtenDeliveryBytes;
	BOOL wantsToKnowWriteStatus;
	BOOL isSecure;
	BOOL showConsole;
	BOOL forcePlainText;
	BOOL delegateKnowsAUT;
	BOOL delegateKnowsDSC;
	BOOL delegateKnowsCWO;
	BOOL delegateKnowsCF;
	BOOL delegateKnowsCC;
	void *sslContext;
	void *sockRLSource;
}

+ (NSArray *)knownSMTPServers;
+ (BOOL)isAddressKnown:(NSString *)address;
+ (NSURL *)serverForAddress:(NSString *)address;
+ (NSString *)userName;
+ (NSArray *)userAddresses;
+ (BOOL)addNewSMTPServer:(NSString *)server user:(NSString *)user path:(NSString *)path password:(NSString *)pass port:(UInt16)port;

- (id)initWithDelegate:(id)aDel;

- (void)setToAddress:(NSString *)address;
- (NSString *)toAddress;
- (void)setToName:(NSString *)name;
- (NSString *)toName;
- (void)setFromAddress:(NSString *)address;
- (NSString *)fromAddress;
- (void)setFromName:(NSString *)name;
- (NSString *)fromName;
- (void)setBody:(id)aBody;
- (id)body;
- (void)setSubject:(NSString *)sub;
- (NSString *)subject;
- (void)setServerURL:(NSURL *)url;
- (NSURL *)serverURL;
- (void)setForcePlainText:(BOOL)flag;
- (BOOL)forcePlainText;
- (void)setShowConsole:(BOOL)flag;
- (BOOL)showConsole;
- (void)setDelegate:(id)aDel;
- (id)delegate;
- (void)setUserName:(NSString *)userName password:(NSString *)password;

- (void)addAttachment:(id)anObj name:(NSString *)name extraHeaders:(NSArray *)extraHeaders;
- (NSArray *)attachments;
- (void)addHeader:(NSString *)aHeader;
- (NSArray *)headers;

- (DSDeliveryStatus)deliveryStatus;

- (void)deliverMessage:(id)sender;

@end

@interface NSObject (DSMailDeliveryDelegate)
- (void)connectionWasOpened:(DSMailDelivery *)mailDelivery;
- (void)connectionFailed:(DSMailDelivery *)mailDelivery;
- (void)connectionClosed:(DSMailDelivery *)mailDelivery;
- (NSData *)attachmentDataForUnknownType:(id)type attachmentName:(NSString **)name mimeType:(NSString **)mimeType;
- (void)deliveryStatusChanged:(DSDeliveryStatus)deliveryStatus info:(NSDictionary *)deliveryInfo;
@end
