//
//  ChunkEnumerator.h
//  HexFiend
//
//  Created by ridiculous_fish on 2/8/06.
//  Copyright 2004-2006 ridiculous_fish. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "ByteSlice.h"

@class ByteArray;

/* A chunk consists of a buffer of data with the given length at the given offset.  A returned chunk is valid until the next chunk is returned or the ChunkEnumerator is destroyed. */
struct Chunk_t {
    const unsigned char *buffer;
    unsigned length;
    unsigned long long offset; //offset into ByteArray of the start of buffer
};

struct ChunkRequest_t {
    struct Chunk_t chunk;
    struct ByteSliceReadRequest readRequest;
    BOOL chunkBufferIsNoCopy;
    BOOL requestIsLive;
};

/* ChunkEnumerator is used for clever enumeration of ByteArrays.  The intent is that it will prefetch chunks. */
@interface ChunkEnumerator : NSObject {
    unsigned char *buffer1;
    unsigned char *buffer2;
    unsigned bufferSize;
    
    struct ChunkRequest_t pendingChunkRequest;
    
    int parity; // a parity of 0 means that buffer1 is potentially used by our client and buffer2 should be queued
    
    NSArray *slices;
    unsigned currentSliceIndex;
    unsigned long long currentSliceOffset;
    
    unsigned long long currentSearchOffset;
    LongRange rangeToSearch;
    
    struct ByteSliceReadRequest request1;
    struct ByteSliceReadRequest request2;
}

- initWithByteArray:(ByteArray *)byteArray searchRange:(LongRange)searchRange;

/* buffer will be nil if there are no more chunks */
- (struct Chunk_t)nextChunk;

/* private */
- (void)queueRequest;

@end
